# Create Figure 5

library(ggplot2)
mutlist <- scan("../data/smc-mutlist.txt", what="character")

source("kde.R")  # obtain from https://faculty.missouri.edu/~kaplandm/code/kde.R
N <- 5000 # how many SMC samples to use for the plots

par(mfrow=c(3,3))
par(mar=c(1,1,1,1))

pdfx <- numeric()
pdfy <- numeric()
length <- numeric()

for (i in 1:length(mutlist)){
  reso <- 2^15
  load(paste0("../output/", i, ".rda"))
  
  diag(mmatOO) <- NA
  diag(mmatMM) <- NA
  
  pdfOO <- kde.botev(mmatOO[1:N,1:N], reso, 0, 1.05*max(mmatOO[1:N,1:N],na.rm=T))
  show(1.05*max(mmatOO,na.rm=T))
  OOmax <- 1.05*max(mmatOO,na.rm=T)
  
  pdfOM <- kde.botev(mmatOM[1:N,1:N], reso, 0, 1.05*max(mmatOM[1:N,1:N],na.rm=T))
  show(1.05*max(mmatOM,na.rm=T))
  OMmax <- 1.05*max(mmatOM,na.rm=T)
  
  pdfMM <- kde.botev(mmatMM[1:N,1:N], reso, 0, 1.05*max(mmatMM[1:N,1:N],na.rm=T))
  show(1.05*max(mmatMM,na.rm=T))
  MMmax <- 1.05*max(mmatMM,na.rm=T)
  
  if (i %in% c(1,2,3,4,7)){
    reso <- 2^6 
  }
  
  load(paste0("../output/disgrormsd-",i, ".rda"))
  
  diag(mmatOO) <- NA
  diag(mmatMM) <- NA
  
  pdfOO2 <- kde.botev(mmatOO[1:N,1:N], reso, 0, 1.05*max(mmatOO[1:N,1:N],na.rm=T))
  OOmax2 <- 1.05*max(mmatOO,na.rm=T)
  
  pdfOM2 <- kde.botev(mmatOM[1:N,1:N], reso, 0, 1.05*max(mmatOM[1:N,1:N],na.rm=T))
  OMmax2 <- 1.05*max(mmatOM,na.rm=T)
  
  pdfMM2 <- kde.botev(mmatMM[1:N,1:N], reso, 0, 1.05*max(mmatMM[1:N,1:N],na.rm=T))
  MMmax2 <- 1.05*max(mmatMM,na.rm=T)
  
  pdfx <- c(pdfx, c(pdfOO[1,], pdfOM[1,], pdfMM[1,], pdfOO2[1,], pdfOM2[1,], pdfMM2[1,]))
  pdfy <- c(pdfy, c(pdfOO[2,], pdfOM[2,], pdfMM[2,], pdfOO2[2,], pdfOM2[2,], pdfMM2[2,]))
  length <- c(length, length(pdfOO[1,]), length(pdfOM[1,]), length(pdfMM[1,]), 
              length(pdfOO2[1,]), length(pdfOM2[1,]), length(pdfMM2[1,]))
}

mutlist <- factor(mutlist, levels=c("D614G", "A222V", "D574Y", "H655Y", "S477N", "T632N", "P272L", "A930V", "D936Y"))
mut_length <- sapply(0:8, function(i) sum(length[(i*6+1):(i*6+6)]))
df <- data.frame(x = pdfx,
                 y = pdfy,
                 category = rep(rep(c("SMC:Reference/Reference","SMC:Reference/Mutated", "SMC:Mutated/Mutated", 
                                      "DiSGro:Reference/Reference","DiSGro:Reference/Mutated", "DiSGro:Mutated/Mutated"), 9), length),
                 mutation = rep(mutlist, mut_length))

df$category <- factor(df$category, levels = c("SMC:Reference/Reference","DiSGro:Reference/Reference","SMC:Reference/Mutated", 
                                              "DiSGro:Reference/Mutated", "SMC:Mutated/Mutated", "DiSGro:Mutated/Mutated"))


p <- ggplot(df, aes(x=x, y = y, color = category)) + 
  geom_line(aes(linetype=category))+ xlab("")+ facet_wrap(~mutation, scales = "free")+
  ylab("Probability Density") + 
  theme(legend.title = element_blank())+
  theme(legend.position="bottom")+
  scale_linetype_manual(values=c("solid","dotted","solid", 
                                 "dotted","solid","dotted"))+
  scale_color_manual(values = c("#E7B800", "#E7B800", "#FC4E07", 
                                "#FC4E07","#00AFBB","#00AFBB"))      


pdf(file="../output/smc_plot.pdf", width = 9, height = 9)  
p
dev.off()
