# Run sampling methods on 3-D S protein structure

# List of mutations to consider
mutlist <- scan("../data/smc-mutlist.txt", what="character")

# Run SMC sampling on each segment
for (i in 1:length(mutlist)) {
  
  mutloc <- as.numeric(substr(mutlist[i],2,4))
  astart <- mutloc-7
  aend <- mutloc+7
  
  cmd1 <- paste0("cd ../petals; ./petals -f ../data/ref-6XM0A.pdb -start ", astart, " -end ", aend, " -seeds 60000 -numloop 30000 -cores 4")
  cmd2 <- paste0("cd ../petals; ./petals -f ../data/", mutlist[i] ,".pdb -start ", astart, " -end ", aend, " -seeds 60000 -numloop 30000 -cores 4")
  
  if (i==3) {
   cmd1 <- paste0(cmd1, " -backgrow")
   cmd2 <- paste0(cmd2, " -backgrow") 
  }
  
  show(cmd1)
  system(cmd1)
  
  show(cmd2)
  system(cmd2)
}

# Subset best 5000 according to energy function
for (i in 1:length(mutlist)) {
  mutloc <- as.numeric(substr(mutlist[i],2,4))
  astart <- mutloc-7
  aend <- mutloc+7  
  
  system(paste0("grep LOOP ../petals/pdbout/ref-6XM_", astart, "_", aend, ".txt > ../output/ref-6XM_", astart, "_", aend, "_energy.txt"))
  system(paste0("grep LOOP ../petals/pdbout/", mutlist[i], "_", astart, "_", aend, ".txt > ../output/", mutlist[i], "_", astart, "_", aend, "_energy.txt"))
         
  # Original sequence
  samps <- readLines(paste0("../petals/pdbout/ref-6XM_", astart, "_", aend, ".txt"))
  energies <- read.table(paste0("../output/ref-6XM_", astart, "_", aend, "_energy.txt"))
  compE <- energies$V5 + 10*energies$V6
  loopids <- grep("LOOP",samps)
  numatoms <- loopids[2]-loopids[1]-1
  best <- order(compE)[1:5000]
  rowSub <- c()
  for (j in 1:length(best)) {
    startRow <- loopids[best[j]]
    rowSub <- append(rowSub, startRow:(startRow+numatoms))
  }
  writeLines(samps[rowSub], paste0("../output/ref-6XM_", astart, "_", aend, "_subset.txt"))
  
  # Mutant sequence
  samps <- readLines(paste0("../petals/pdbout/", mutlist[i], "_", astart, "_", aend, ".txt"))
  energies <- read.table(paste0("../output/", mutlist[i], "_", astart, "_", aend, "_energy.txt"))
  compE <- energies$V5 + 10*energies$V6
  loopids <- grep("LOOP",samps)
  numatoms <- loopids[2]-loopids[1]-1
  best <- order(compE)[1:5000]
  rowSub <- c()
  for (j in 1:length(best)) {
    startRow <- loopids[best[j]]
    rowSub <- append(rowSub, startRow:(startRow+numatoms))
  }
  writeLines(samps[rowSub], paste0("../output/", mutlist[i], "_", astart, "_", aend, "_subset.txt"))
}

# Assess SMC prediction accuracy compared to original PDB sequence
for (i in 1:length(mutlist)) {
  mutloc <- as.numeric(substr(mutlist[i],2,4))
  astart <- mutloc-7
  aend <- mutloc+7
  
  cmd1 <- paste0("cd ../petals; ./petals -f ../data/ref-6XM0A.pdb -start ", astart, " -end ", aend, " -mode compare -score ../output/ref-6XM_", astart, "_", aend, "_subset.txt")
  show(cmd1)
  system(cmd1) # RMSD of prediction is 9th number in output line
}

# Extract and save backbone coords from SMC samples
for (i in 1:length(mutlist)) {
  mutloc <- as.numeric(substr(mutlist[i],2,4))
  astart <- mutloc-7
  aend <- mutloc+7
  
  mlist <- list()  
  
  # Mutants
  samps <- readLines(paste0("../output/", mutlist[i], "_", astart, "_", aend, "_subset.txt"))
  loopids <- grep("LOOP",samps)
  numatoms <- loopids[2]-loopids[1]
  
  for (ind in 1:length(loopids)) {
    li <- samps[ (loopids[ind]+1):(loopids[ind]+numatoms-1) ]

    mlist[[ind]] <- matrix(NA, nrow=0, ncol=3)
    for ( j in 1:length(li)) {
      if ( substr(li[j],14,15) %in% c("N ", "CA", "C ", "O ")) {
        xyz <- as.numeric(strsplit(substr(li[j],28,55), "\\s+")[[1]])
        xyz <- xyz[!is.na(xyz)]
        mlist[[ind]] <- rbind(mlist[[ind]], xyz)
      }
    }
  }
  save(mlist, file=paste0("../output/", mutlist[i], "_", astart, "_", aend,  "_subset_backbone_xyz.rda"))
  
  olist <- list()  
  
  # Original
  samps <- readLines(paste0("../output/ref-6XM_", astart, "_", aend,  "_subset.txt"))
  loopids <- grep("LOOP",samps)
  numatoms <- loopids[2]-loopids[1]
  
  for (ind in 1:length(loopids)) {
    
    li <- samps[ (loopids[ind]+1):(loopids[ind]+numatoms-1) ]
    olist[[ind]] <- matrix(NA, nrow=0, ncol=3)
    for ( j in 1:length(li)) {
      if ( substr(li[j],14,15) %in% c("N ", "CA", "C ", "O ")) {
        xyz <- as.numeric(strsplit(substr(li[j],28,55), "\\s+")[[1]])
        xyz <- xyz[!is.na(xyz)]
        olist[[ind]] <- rbind(olist[[ind]], xyz)
      }
    }
  }
  save(olist, file=paste0("../output/ref-6XM_", astart, "_", aend, "_subset_backbone_xyz.rda"))  
}

# Create pairwise RMSD matrices for SMC samples
for (i in 1:length(mutlist)) {
  mutloc <- as.numeric(substr(mutlist[i],2,4))
  astart <- mutloc-7
  aend <- mutloc+7
  
  load(paste0("../output/", mutlist[i], "_", astart, "_", aend,  "_subset_backbone_xyz.rda"))
  load(paste0("../output/ref-6XM_", astart, "_", aend, "_subset_backbone_xyz.rda"))
  mmatOO <- matrix(NA, nrow = length(olist), ncol=length(olist))
  for (ii in 1:length(olist)) {
    #show(ii)
    for (jj in 1:length(olist)) {
      mmatOO[ii,jj] <-  sqrt(mean((olist[[ii]]-olist[[jj]])^2)*3)
    }  
  }
  
  mmatOM <- matrix(NA, nrow = length(olist), ncol=length(mlist))
  for (ii in 1:length(olist)) {
    #show(ii)
    for (jj in 1:length(mlist)) {
      mmatOM[ii,jj] <-  sqrt(mean((olist[[ii]]-mlist[[jj]])^2)*3)
    }  
  }
  
  # mmatOM[1,1] is the pairwise RMSD between the lowest energy conformations of original/mutated SMC samples
  show(mmatOM[1:5,1:5])  
  
  mmatMM <- matrix(NA, nrow = length(mlist), ncol=length(mlist))
  for (ii in 1:length(mlist)) {
    #show(ii)
    for (jj in 1:length(mlist)) {
      mmatMM[ii,jj] <-  sqrt(mean((mlist[[ii]]-mlist[[jj]])^2)*3)
    }  
  }

  save(mmatOO, mmatOM, mmatMM, file=paste0("../output/", i, ".rda"))
}


# Run Disgro sampling on each segment
for (i in 1:length(mutlist)) {
  
  mutloc <- as.numeric(substr(mutlist[i],2,4))
  astart <- mutloc-7
  aend <- mutloc+7
  
  cmd1 <- paste0("cd ../Disgro; ./disgro -mode smc -f ../pdbs/ref-6XM0A.pdb -start ", astart, " -end ", aend, " -n 100000 -eval -confkeep 100000 -nscc 20 -pdbout 5000 &")
  cmd2 <- paste0("cd ../Disgro; ./disgro -mode smc -f ../pdbs/", mutlist[i] ,".pdb -start ", astart, " -end ", aend, " -n 100000 -eval -confkeep 100000 -nscc 20 -pdbout 5000 &")
  
  show(cmd1)
  system(cmd1)
  
  show(cmd2)
  system(cmd2)
}

# Evaluate energy and RMSD of Disgro samples
for (ii in 1:length(mutlist)) {
  
  mutloc <- as.numeric(substr(mutlist[ii],2,4))
  astart <- mutloc-7
  aend <- mutloc+7
  
  eVector <- matrix(NA, ncol=3, nrow=5000)
  for (i in 1:5000) {
    fname <- paste("../Disgro/pdb_output/ref-_", astart, "_", aend, "_", i, "topconf.pdb", sep = "")
    if (!is.na(file.info(fname)$size)) {
      cmdsc <- paste("cd ../petals; ./petals -mode energy -f ", fname, " -start ", astart, " -end ", aend, " -native ../data/ref-6XM0A.pdb", sep = "")
      cat(cmdsc, "\n")
      eVector[i,] <- as.numeric(unlist(strsplit(system(cmdsc, intern=T)," ")))
    }
  }
  
  show(eVector[which.min(eVector[,1]),3])  # RMSD of Disgro prediction
  
  save(cmdsc, eVector, file=paste0("../output/disgro-", ii, ".rda"))
}

# Extract and save backbone coords for Disgro samples
for (i in 1:length(mutlist)) {
  mutloc <- as.numeric(substr(mutlist[i],2,4))
  astart <- mutloc-7
  aend <- mutloc+7
  
  mlist <- list()  
  
  # Mutants
  for (ind in 1:5000) {
    fname <- paste("../Disgro/pdb_output/", substr(mutlist[i],1,4), "_", astart, "_", aend, "_", ind, "topconf.pdb", sep = "")
    if (!is.na(file.info(fname)$size)) {
      li <- readLines(fname)
      mlist[[ind]] <- matrix(NA, nrow=0, ncol=3)
      for (j in 1:length(li)) {
        resid <- as.numeric(substr(li[j],23,27))
        if ( substr(li[j],14,15) %in% c("N ", "CA", "C ", "O ") & resid >= astart & resid <= aend ) {
          xyz <- as.numeric(strsplit(substr(li[j],28,55), "\\s+")[[1]])
          xyz <- xyz[!is.na(xyz)]
          mlist[[ind]] <- rbind(mlist[[ind]], xyz)
        }
        
      }
    }
  }
  
  save(mlist, file=paste0("../output/disgro_", mutlist[i], "_", astart, "_", aend,  "_subset_backbone_xyz.rda"))
  
  olist <- list()  
  
  # Original
  for (ind in 1:5000) {
    fname <- paste("../Disgro/pdb_output/ref-_", astart, "_", aend, "_", ind, "topconf.pdb", sep = "")
    if (!is.na(file.info(fname)$size)) {
      li <- readLines(fname)
      olist[[ind]] <- matrix(NA, nrow=0, ncol=3)
      for (j in 1:length(li)) {
        resid <- as.numeric(substr(li[j],23,27))
        if ( substr(li[j],14,15) %in% c("N ", "CA", "C ", "O ") & resid >= astart & resid <= aend ) {
          xyz <- as.numeric(strsplit(substr(li[j],28,55), "\\s+")[[1]])
          xyz <- xyz[!is.na(xyz)]
          olist[[ind]] <- rbind(olist[[ind]], xyz)
        }
        
      }
    }
  }

  save(olist, file=paste0("../output/disgro_ref-6XM_", astart, "_", aend, "_subset_backbone_xyz.rda"))  
}

# Create pairwise RMSD matrices for Disgro samples
for (i in 1:length(mutlist)) {
  mutloc <- as.numeric(substr(mutlist[i],2,4))
  astart <- mutloc-7
  aend <- mutloc+7
  
  load(paste0("../output/disgro_", mutlist[i], "_", astart, "_", aend,  "_subset_backbone_xyz.rda"))
  load(paste0("../output/disgro_ref-6XM_", astart, "_", aend, "_subset_backbone_xyz.rda"))
  mmatOO <- matrix(NA, nrow = length(olist), ncol=length(olist))
  for (ii in 1:length(olist)) {
    #show(ii)
    for (jj in 1:length(olist)) {
      mmatOO[ii,jj] <-  sqrt(mean((olist[[ii]]-olist[[jj]])^2)*3)
    }  
  }
  
  mmatOM <- matrix(NA, nrow = length(olist), ncol=length(mlist))
  for (ii in 1:length(olist)) {
    #show(ii)
    for (jj in 1:length(mlist)) {
      mmatOM[ii,jj] <-  sqrt(mean((olist[[ii]]-mlist[[jj]])^2)*3)
    }  
  }
  
  mmatMM <- matrix(NA, nrow = length(mlist), ncol=length(mlist))
  for (ii in 1:length(mlist)) {
    #show(ii)
    for (jj in 1:length(mlist)) {
      mmatMM[ii,jj] <-  sqrt(mean((mlist[[ii]]-mlist[[jj]])^2)*3)
    }  
  }
  
  # Coerce to 5000x5000 matrix
  tmpmmatOO <- matrix(NA, nrow=5000,ncol=5000)
  tmpmmatOO[1:nrow(mmatOO),1:nrow(mmatOO)] <- mmatOO
  mmatOO <- tmpmmatOO

  tmpmmatOM <- matrix(NA, nrow=5000,ncol=5000)
  tmpmmatOM[1:nrow(mmatOM),1:ncol(mmatOM)] <- mmatOM
  mmatOM <- tmpmmatOM
  
  # mmatOM[1,1] is the pairwise RMSD between the lowest energy conformations of original/mutated DisGro samples
  show(mmatOM[1:5,1:5])  
  
  tmpmmatMM <- matrix(NA, nrow=5000,ncol=5000)
  tmpmmatMM[1:nrow(mmatMM),1:nrow(mmatMM)] <- mmatMM
  mmatMM <- tmpmmatMM

  save(mmatOO, mmatOM, mmatMM, file=paste0("../output/disgrormsd-", i, ".rda"))
  
}

